// RotateRoutine.cpp: implementation of the CRotateRoutine class.
//
// Author      : Sebastian Ledesma
//
//////////////////////////////////////////////////////////////////////

#include "CTwirlRoutine.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
#include <math.h>

CTwirlRoutine::CTwirlRoutine() {
	m_iHeight = 0;
	m_iWidth  = 0;
   step=0;
   direccion=1;
   maxTwirl=16; //try with another values, like 24, 32 or 48
}

CTwirlRoutine::~CTwirlRoutine() {
}


void CTwirlRoutine::Create(int iWidth,int iHeight) {
	m_iHeight = iHeight;
	m_iWidth  = iWidth;

	// Init the Cost Table
	InitCostBLTable();
}


/* ------------------------------------------------------------------------
   Generate look-up tables for sine and cosine. The wave is stretched to
   fill 512 (0..511) positions, so that position 255 is about 180 degrees.
   This give us greater accuracy and speed, since the index can be
   ANDed with 512. No explicit range checking is necessary:

     sin_of_some_angle = tsin[some_angle & 0x1ff];
   ------------------------------------------------------------------------ */

void CTwirlRoutine::InitCostBLTable() {
//Remember: 180  = Pi [radian]
//and sin, cos, etc, all expect angles expressed in radians.
const float factor = 3.1415926535897932384626433832795 / 256.0;
//const float Pi = 3.1415926535897932384626433832795;
	for (int c=0; c<512; ++c) {
		//tsin[c] = sin( c*360/512 * Pi / 180 );
		//tcos[c] = cos( c*360/512 * Pi / 180 );
    //if we do the algebraic work, we can use this expression that's faster
		tsin[c] = sin( c* factor );
		tcos[c] = cos( c* factor );
	}
}

void CTwirlRoutine::InitPallette(){
}


void CTwirlRoutine::Calculate() {
	step=step+direccion;
	if (step>=maxTwirl)
		direccion=-1;
	else if (step<=0)//or step<= -maxTwirl or 
		direccion=1;
}


void CTwirlRoutine::Render(DWORD* pSrcImage, DWORD* pTargetImage) {
DWORD *pDest, *pSrc;

int  x,y, x2,y2, xCenter, yCenter, angle;
int b, ysqr;
float cosy,siny;
float actdist,dist;

	xCenter = m_iWidth / 2;
	yCenter = m_iHeight / 2;
	dist = sqrt(xCenter^xCenter + yCenter^yCenter);

	b= step*8;
	for (y=0;  y< m_iHeight; y++) {
		pDest = &pTargetImage[m_iWidth * y];
		ysqr = (y-yCenter)*(y-yCenter);
      for (x=0; x<m_iWidth; x++) {
	      actdist = sqrt((x-xCenter)*(x-xCenter)+ysqr)/dist;

	      actdist = tsin[ ((int)(actdist*1024)) & 0x1ff ];
   	   angle = 384 + actdist * b ;

	      cosy = (y-yCenter) * tcos[angle & 0x1ff];
   	   siny = (y-yCenter) * tsin[angle & 0x1ff];

	      x2 = ((x-xCenter) * tsin[angle & 0x1ff] + cosy) + xCenter;
   	   y2 = ((x-xCenter) * tcos[angle & 0x1ff] - siny) + yCenter;
			if ( y2>=0 && y2<m_iHeight && x2>=0 && x2<m_iWidth ) {
				pSrc = &pSrcImage[m_iWidth * y2];
				pDest[x] = pSrc[m_iWidth-x2];
			}
         else {
	        if (pDest[x] > 2)
   	       pDest[x] = pDest[x]-2;
      	  else
         	 pDest[x] = 0;
	      }
		}
	}

   Calculate();
}

