//----------------------------------------------------------------------------
//  Project SpecialFX
//  
//  Copyright  2009. All Rights Reserved.
//
//  SUBSYSTEM:    SpecialFX Application
//  FILE:         specialfxdlgclient.cpp
//  AUTHOR:       
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TSpecialFXDlgClient (TDialog).
//
//----------------------------------------------------------------------------

#include <owl/pch.h>
//#include <winsys/wsyscls.h>
#include "aboutdlg.h"

#include "specialfxapp.h"
#include "specialfxdlgclient.h"


//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(TSpecialFXDlgClient, TDialog)
//{{TSpecialFXDlgClientRSP_TBL_BEGIN}}
  EV_BN_CLICKED(IDC_ANIMATE, OnAnimate),
  EV_BN_CLICKED(IDC_STOP, OnStop),
  EV_BN_CLICKED(IDC_CHECKBOX_FIRE, OnCheckFire),
  EV_BN_CLICKED(IDC_CHECKBOX_PLASMA, OnCheckPlasma),
  EV_BN_CLICKED(IDC_CHECKBOX_WATER, OnCheckWater),
  EV_SB_THUMBPOSITION( IDC_TRACKBAR_FIRE,  OnSlideFire),
  EV_SB_THUMBPOSITION( IDC_TRACKBAR_PLASMA,OnSlidePlasma),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_TRACKBAR_FIRE,   OnTrackAll_Fire),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_TRACKBAR_PLASMA, OnTrackAll_Plasma),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_TRACKBAR_TWIRL, OnTrackAll_Twirl),
  EV_WM_TIMER,
  EV_WM_MOUSEMOVE,
  EV_WM_LBUTTONDOWN,
  EV_WM_PAINT,
  EV_BN_CLICKED(IDC_ABOUT, OnAbout),
  EV_BN_CLICKED(IDC_CHECKBOX_ROTATE, OnCheckRotate),
  EV_BN_CLICKED(IDC_CHECKBOX_SNOW, OnCheckSnow),
  EV_BN_CLICKED(IDC_CHECKBOX_TWIRL, OnCheckTwirl),
//{{TSpecialFXDlgClientRSP_TBL_END}}
END_RESPONSE_TABLE;


//{{TSpecialFXDlgClient Implementation}}


//--------------------------------------------------------
// TSpecialFXDlgClient
// ~~~~~~~~~~
// Construction/Destruction handling.
//

TPoint bitmapPosition(15,20);

TSpecialFXDlgClient::TSpecialFXDlgClient(TWindow* parent, TResId resId, TModule* module)
:  TDialog(parent, resId, module)
{
//{{TSpecialFXDlgClientXFER_USE}}
  pCheckBox_Fire = new TCheckBox(this, IDC_CHECKBOX_FIRE, 0);
  pCheckBox_Plasma = new TCheckBox(this, IDC_CHECKBOX_PLASMA, 0);
  pCheckBox_Water = new TCheckBox(this, IDC_CHECKBOX_WATER, 0);
  pTrack_Fire = new THSlider(this, IDC_TRACKBAR_FIRE, IDB_HSLIDERTHUMB);
  pTrack_Plasma = new THSlider(this, IDC_TRACKBAR_PLASMA, IDB_HSLIDERTHUMB);
  pCheckBox_Rotate = new TCheckBox(this, IDC_CHECKBOX_ROTATE, 0);
  pCheckBox_Twirl = new TCheckBox(this, IDC_CHECKBOX_TWIRL, 0);
  pTrack_Twirl = new THSlider(this, IDC_TRACKBAR_TWIRL, IDB_HSLIDERTHUMB);
  pCheckBox_Snow = new TCheckBox(this, IDC_CHECKBOX_SNOW, 0);

//{{TSpecialFXDlgClientXFER_USE_END}}

  // INSERT>> Your constructor code here.
	runWater=runFire=runPlasma=runRotate=runTwirl=runSnow=false;

	bmpRenderSource=NULL;
   bmpRenderTarget=NULL;
	pTrack_Fire->SetRuler(1,false);
	pTrack_Plasma->SetRuler(1,false);
   pTrack_Twirl->SetRuler(1,false);
}


TSpecialFXDlgClient::~TSpecialFXDlgClient(){
	if (bmpRenderSource)
   	delete bmpRenderSource;
   if (bmpRenderTarget)
   	delete bmpRenderTarget;
  Destroy();
  // INSERT>> Your destructor code here.
}

void TSpecialFXDlgClient::SetupWindow(){
  TDialog::SetupWindow();
	// INSERT>> Your code here.
//	TBitmap myTmpPicture(GetModule()->GetInstance(), TResId(IDB_BITMAP1));
	TBitmap myTmpPicture(GetModule()->GetHandle(), TResId(IDB_BITMAP1));
	bmpRenderSource = new TDib(myTmpPicture);
	bmpRenderTarget = new TDib(myTmpPicture); //now each has a own version

	// 'create' water object
	myWater.Create(bmpRenderSource->Width(), bmpRenderSource->Height());

	// 'create' our fire object
	myFire.Create(bmpRenderSource->Width(), bmpRenderSource->Height());
	myFire.m_iAlpha = 30;  //can take values from 0 to 255
	pTrack_Fire->SetRange(1,192); //plasma alfa limits
	pTrack_Fire->SetPosition(30);

	// now the plasma
	myPlasma.Create( bmpRenderSource->Width(), bmpRenderSource->Height());
	myPlasma.m_iAlpha = 30;  //can take values from 0 to 255
	pTrack_Plasma->SetRange(1,192); //
	pTrack_Plasma->SetPosition(30);

   // the Rotate effect
	myRotateEffect.Create(bmpRenderSource->Width(), bmpRenderSource->Height());

   // the TWirl effect
	myTwirlEffect.Create(bmpRenderSource->Width(), bmpRenderSource->Height());
	pTrack_Twirl->SetRange(1,64); //
	pTrack_Twirl->SetPosition(16);

   // reserved for you:
   //mySnowEffect.Create(bmpRenderSource->Width(), bmpRenderSource->Height());

}


void TSpecialFXDlgClient::CleanupWindow() {
	KillTimer(555);
	TDialog::CleanupWindow();
}


void TSpecialFXDlgClient::OnAnimate(){
	SetTimer(555, 30);
}


void TSpecialFXDlgClient::OnStop() {
	KillTimer(555);
}


void TSpecialFXDlgClient::OnCheckFire(){
  runFire = pCheckBox_Fire->GetCheck()==BF_CHECKED;
  /*if (!runFire)
	  myFire.Reset();  */
}

void TSpecialFXDlgClient::OnCheckPlasma(){
  runPlasma = pCheckBox_Plasma->GetCheck()==BF_CHECKED;
}

void TSpecialFXDlgClient::OnCheckWater(){
	runWater = pCheckBox_Water->GetCheck()==BF_CHECKED;
}

void TSpecialFXDlgClient::OnCheckRotate() {
  // INSERT>> Your code here.
	runRotate=pCheckBox_Rotate->GetCheck()==BF_CHECKED;
}

void TSpecialFXDlgClient::OnCheckTwirl() {
  // INSERT>> Your code here.
	runTwirl=pCheckBox_Twirl->GetCheck()==BF_CHECKED;
}

void TSpecialFXDlgClient::OnCheckSnow() {
  // INSERT>> Your code here.
	runSnow=pCheckBox_Snow->GetCheck()==BF_CHECKED;
   if (runSnow)
   	pCheckBox_Snow->SetCaption("Snow: this is your homework");
   else
   	pCheckBox_Snow->SetCaption("Snow");
}


void TSpecialFXDlgClient::OnSlideFire(){
	myFire.m_iAlpha = pTrack_Fire->GetPosition();
}

void TSpecialFXDlgClient::OnSlidePlasma(){
	myPlasma.m_iAlpha = pTrack_Plasma->GetPosition();
}

void TSpecialFXDlgClient::OnTrackAll_Fire(uint code){
	switch (code) {
		case SB_PAGEUP:
		case SB_PAGEDOWN:
   	case SB_THUMBTRACK:
			//leer posicion y actualizar
			myFire.m_iAlpha = pTrack_Fire->GetPosition();
         break;

		case SB_ENDSCROLL:
			//myFire.m_iAlpha = pTrack_Fire->GetPosition();
		  	break;
		default:
		  break;
	}
}

void TSpecialFXDlgClient::OnTrackAll_Plasma(uint code){
	switch (code) {
		case SB_PAGEUP:
		case SB_PAGEDOWN:
   	case SB_THUMBTRACK:
			//leer posicion y actualizar
			//myPlasma.m_iAlpha = pTrack_Plasma->GetPosition();
         break;

		case SB_ENDSCROLL:
			myPlasma.m_iAlpha = pTrack_Plasma->GetPosition();
		  	break;
		default:
		  break;
	}
}


void TSpecialFXDlgClient::OnTrackAll_Twirl(uint code){
	switch (code) {
		case SB_PAGEUP:
		case SB_PAGEDOWN:
   	case SB_THUMBTRACK:
			//leer posicion y actualizar
			myTwirlEffect.maxTwirl = pTrack_Twirl->GetPosition();
         break;

		case SB_ENDSCROLL:
			myTwirlEffect.maxTwirl = pTrack_Twirl->GetPosition();
		  	break;
		default:
		  break;
	}
}

void TSpecialFXDlgClient::EvPaint() {
    TDialog::EvPaint();

  // INSERT>> Your code here.
//	TRect r=GetClientRect();
//	TPaintDC  dc(*this);
//	Paint(dc, false, r);

		TRect r;
		r.left = bitmapPosition.x;
		r.top = bitmapPosition.y;
		r.right = r.left + bmpRenderTarget->Width();
		r.bottom = r.top + bmpRenderTarget->Height();

	   HDC hdc=GetDC(GetHandle());
      if (hdc) {
			TDC dc(hdc);
	      dc.SetDIBitsToDevice(r, TPoint(0,0), *bmpRenderTarget);
      }

}

/*
void TSpecialFXDlgClient::Paint(TDC& dc, bool erase, TRect& rect)
{
  TDialog::Paint(dc, erase, rect);

  // INSERT>> Your code here.
	TRect r;
	r.left = bitmapPosition.x;
	r.top = bitmapPosition.y;
	r.right = r.left + bmpRenderTarget->Width();
	r.bottom = r.top + bmpRenderTarget->Height();
	dc.SetDIBitsToDevice(r, TPoint(0,0), *bmpRenderTarget);

} */


void TSpecialFXDlgClient::EvTimer(uint timerId){
	TDialog::EvTimer(timerId);

	// INSERT>> Your code here.
	if(timerId == 555) {
   	//Transform effects
		if(!runWater) {
		 	myWater.FlattenWater();// flatten it so it appears we turned it off...*
         if (runRotate)
				myRotateEffect.Render((DWORD *)bmpRenderSource->GetBits(),(DWORD *)bmpRenderTarget->GetBits());
         else if (runTwirl)
				myTwirlEffect.Render((DWORD *)bmpRenderSource->GetBits(),(DWORD *)bmpRenderTarget->GetBits());
         else
				myWater.Render((DWORD *)bmpRenderSource->GetBits(),(DWORD*)bmpRenderTarget->GetBits());
      }
      else
			myWater.Render((DWORD *)bmpRenderSource->GetBits(),(DWORD*)bmpRenderTarget->GetBits());

      //Filter effects
		if(runFire)
			myFire.Render((DWORD *)bmpRenderTarget->GetBits());

		if(runPlasma)
			myPlasma.Render((DWORD *)bmpRenderTarget->GetBits());

      if (runSnow)
      	; //mySnow.Render(bmpRenderTarget->GetBits());

      //Now draw
		TRect r;
		r.left = bitmapPosition.x;
		r.top = bitmapPosition.y;
		r.right = r.left + bmpRenderTarget->Width();
		r.bottom = r.top + bmpRenderTarget->Height();

		HDC hdc=GetDC(GetHandle());
		if (hdc) {
			TDC dc(hdc);
			dc.SetDIBitsToDevice(r, TPoint(0,0), *bmpRenderTarget);
		}

	}
}


void TSpecialFXDlgClient::EvMouseMove(uint modKeys, TPoint& point)
{
    TDialog::EvMouseMove(modKeys, point);

  // INSERT>> Your code here.
	TRect r;
	r.left = bitmapPosition.x;
	r.top = bitmapPosition.y;
	r.right = r.left + bmpRenderTarget->Width();
	r.bottom = r.top + bmpRenderTarget->Height();

	if(r.Contains(point) == true) 	{
		// since dibs are drawn upside down we need to flip the y position (for it to look right)
		point.y -= bitmapPosition.y;// offset from top of screen to image..
		point.y = bmpRenderTarget->Height()-point.y;

		myWater.HeightBlob(point.x-bitmapPosition.x, point.y, 5,50, myWater.m_iHpage);
	}
}


void TSpecialFXDlgClient::EvLButtonDown(uint modKeys, TPoint& point){
    TDialog::EvLButtonDown(modKeys, point);

  // INSERT>> Your code here.
	TRect r;
	r.left = bitmapPosition.x;
	r.top = bitmapPosition.y;
	r.right = r.left + bmpRenderTarget->Width();
	r.bottom = r.top + bmpRenderTarget->Height();

	if(r.Contains(point) == true)	{
		// since dibs are drawn upside down we need to flip the y position (for it to look right)
		point.y -= bitmapPosition.y;// offset from top of screen to image..
		point.y = bmpRenderTarget->Height()-point.y;

		myWater.HeightBlob(point.x-bitmapPosition.x, point.y, 10, 500, myWater.m_iHpage);
	}
}




void TSpecialFXDlgClient::OnAbout() {
	TAboutDlg   xTAboutDlg(this);
	xTAboutDlg.Execute();
}






